---
jupyter:
  jupytext:
    text_representation:
      extension: .md
      format_name: markdown
      format_version: '1.3'
      jupytext_version: 1.13.1
  kernelspec:
    display_name: Python 3 (ipykernel)
    language: python
    name: python3
---

```python
from itertools import cycle
import os

import numpy as np
np.warnings.filterwarnings('ignore') # silence numpys warnings

from matplotlib import pyplot as plt
%matplotlib inline
from matplotlib.colors import TABLEAU_COLORS as COLORS

from pymor.core.logger import set_log_levels, getLogger

set_log_levels({
    'main': 'DEBUG',
    'pymor': 'WARN',
})
logger = getLogger('main.main')
```

## Available finished runs

```python
!ls -lh ./data_figures/
```

```python
# Results from the latest restarts which I started on 20.02.23:
list_folders = ['ianscluster03_ANN', 'ianscluster04_ANN', 'ianscluster05_SDKN', 
                'ianscluster07_2L-VKOGA', 'ianscluster08_VKOGA']
list_methods = ['ANN_200', 'ANN_200', 'SDKN_200', 'VKOGA_40', 'VKOGA_40']
```

## Load a run

```python
from pymor.core.pickle import load


# Specify with run to load
idx_method = 4


with open(f'./data_figures/' + list_folders[idx_method] 
          + '/old_MC_adaptive_model_1_1000_5.00e-02_' + list_methods[idx_method] 
          + '_CURRENT_RUN.pickle', 'rb') as file:
    results, statistics = load(file)
```

## Some analysis to extract performance quantities

```python
# Check which model was used
list_flags_FOM = [True if str == 'FOM' else False for str in statistics['models']]
list_flags_ROM = [True if str == 'ROM' else False for str in statistics['models']]
list_flags_MLM = [True if str == 'MLM' else False for str in statistics['models']]

array_indices = np.arange(1, len(statistics['timings']['eval_model']) + 1)
array_elapsed = np.array(statistics['timings']['eval_model'])
```

```python
# Cut to a predescribed amount of mu evaluations
N_mu = 2000

if N_mu > len(list_flags_FOM):
    N_mu = len(list_flags_FOM)

list_flags_FOM = list_flags_FOM[:N_mu]
list_flags_ROM = list_flags_ROM[:N_mu]
list_flags_MLM = list_flags_MLM[:N_mu]

array_indices = array_indices[:N_mu]
array_elapsed = array_elapsed[:N_mu]
```

```python
## Plot of time-to-output query
plt.figure()
plt.title('time-to-output per query')
plt.xlabel('num outputs requested')
plt.ylabel('time (s)')
plt.semilogy(array_indices[list_flags_FOM], array_elapsed[list_flags_FOM], 'b.')
plt.semilogy(array_indices[list_flags_ROM], array_elapsed[list_flags_ROM], 'g.')
plt.semilogy(array_indices[list_flags_MLM], array_elapsed[list_flags_MLM], 'r.')
plt.legend()
# plt.show()
```

```python
# Compute average time for FOM / ROM / MLM
avg_time_fom = np.mean(array_elapsed[list_flags_FOM])
avg_time_rom = np.mean(array_elapsed[list_flags_ROM])
avg_time_mlm = np.mean(array_elapsed[list_flags_MLM])

print('Average FOM solve time (# = {:4d}): {:.3f}s.'.format(np.sum(list_flags_FOM), avg_time_fom))
print('Average ROM solve time (# = {:4d}): {:.3f}s.'.format(np.sum(list_flags_ROM), avg_time_rom))
print('Average MLM solve time (# = {:4d}): {:.3f}s.'.format(np.sum(list_flags_MLM), avg_time_mlm))

```

```python
## Check when training took place
list_idx_training = []

for idx in range(len(statistics['timings']['build_model'])):
    if idx > N_mu:
        continue
    
    value = statistics['timings']['build_model'][idx]
    
    if type(statistics['timings']['build_model'][idx]) == type((1, 2)):
        continue # here we have FOM!
    if value is None:
        continue # here we have MLM!
    if value > 1:
        print('Training took place for idx_mu = {} and took {:.3f}s.'.format(idx, value))
        
        list_idx_training.append(idx)
```

```python
## Compute ratio of MLM solves
idx0 = 0

# For all the intermediate intervals
for idx in list_idx_training:
    n_MLM = sum(list_flags_MLM[idx0:idx])
    n_ROM = sum(list_flags_ROM[idx0:idx])
    
    print('MLM ratio = {:.3f}%.'.format(100 * n_MLM / (n_MLM + n_ROM)))
    
    idx0 = idx+1
    
# For the final (last) interval
n_MLM = sum(list_flags_MLM[list_idx_training[-1]:])
n_ROM = sum(list_flags_ROM[list_idx_training[-1]:])
    
print('MLM ratio = {:.3f}%.'.format(100 * n_MLM / (n_MLM + n_ROM)))

```

## Produce the plots from the paper

```python
from matplotlib.colors import to_rgba, TABLEAU_COLORS as COLORS

timings_FOM_color = to_rgba('#74a9cf', 1.)
timings_ROM_color = (to_rgba('#fd8d3c', 1.), to_rgba('#fecc5c', 1.))
timings_MLM_color = (to_rgba('#006837', 1.), to_rgba('#78c679', 1.))

```

```python
import matplotlib
# matplotlib.use('Agg')
import tikzplotlib

big_width = 3
MLM_eval_times = []
MLM_build_times = []


for N_plot in (5000,):
    if N_plot > N_mu:
        N_plot = N_mu
    
    list_idx_NN_training = []
    plt.figure()
    # plt.figure(facecolor='white')
    # plt.title('time spent (s) in model evaluation (light) and model building (dark)')
    plt.plot([], [], color=timings_FOM_color, label='FOM')
    plt.plot([], [], color=timings_ROM_color[1], label='RB-ROM')
    plt.plot([], [], color=timings_MLM_color[1], label='ML-ROM')
    # plt.xscale('log')
    plt.yscale('log')
    plt.legend()

    # MLM
    _global_index = [-1]
    for idx in range(N_plot):
        used_model = statistics['models'][idx]
        _global_index[0] += 1
        eval_time = statistics['timings']['eval_model'][idx]
        build_time = statistics['timings']['build_model'][idx]
        if used_model == 'MLM':
            MLM_eval_times.append(eval_time)
            plt.bar(
                _global_index, eval_time, width=1., bottom=1,
                align='edge', color=timings_MLM_color[1])

    # RB
    _global_index = [-1]
    for idx in range(N_plot):
        used_model = statistics['models'][idx]
        _global_index[0] += 1
        eval_time = statistics['timings']['eval_model'][idx]
        build_time = statistics['timings']['build_model'][idx]
        if used_model == 'ROM':
            if build_time > 100: # MLM training
                list_idx_NN_training.append((idx, build_time))
                print(idx)
                print(build_time)
                width = big_width
            else:  # MLM data append
                width = big_width
            plt.bar(
                _global_index, eval_time, width=width, bottom=1,
                align='edge', color=timings_ROM_color[1])
            plt.bar(
                _global_index, build_time, width=width, bottom=eval_time+1,
                align='edge', color=timings_MLM_color[0])

    # FOM
    _global_index = [-1]
    for idx in range(N_plot):
        used_model = statistics['models'][idx]
        _global_index[0] += 1
        eval_time = statistics['timings']['eval_model'][idx]
        build_time = statistics['timings']['build_model'][idx]
        if used_model == 'FOM':
            plt.bar(
                _global_index, eval_time, width=big_width, bottom=1,
                align='edge', color=timings_FOM_color)
            plt.bar(
                _global_index, build_time[0], width=big_width, bottom=eval_time+1,
                align='edge', color=timings_ROM_color[0])
            if build_time[2]:
                plt.bar(
                    _global_index, build_time[1 ] +build_time[2], width=big_width, bottom=eval_time +build_time[0]+1,
                    align='edge', color=timings_MLM_color[0])

    # Save the figure
    plt.savefig('sdkn_for_rbmlrom/' + list_methods[idx_method] + '_time_to_output_{}.png'.format(N_plot),  dpi = 1000, transparent=False)
    tikzplotlib.save('sdkn_for_rbmlrom/' + list_methods[idx_method] + '_time_to_output_{}.tex'.format(N_plot))

```
