import numpy as np

from pymor.analyticalproblems.domaindescriptions import LineDomain
from pymor.analyticalproblems.instationary import InstationaryProblem
from pymor.analyticalproblems.elliptic import StationaryProblem
from pymor.analyticalproblems.functions import ConstantFunction, ExpressionFunction, GenericFunction
from pymor.discretizers.dunegdt.cg import discretize_instationary_cg
from pymor.parameters.functionals import ExpressionParameterFunctional, ProjectionParameterFunctional
from pymor.parameters.functionals import (
    ConstantParameterFunctional,
    MinThetaParameterFunctional,
    GenericParameterFunctional,
    ProjectionParameterFunctional,
)


def make_G0R1_problem(T):
    return InstationaryProblem(stationary_part=StationaryProblem(

        domain=LineDomain([0, 1], left='dirichlet', right='neumann'),

        rhs=ConstantFunction(0, dim_domain=1),

        diffusion=ConstantFunction(1, dim_domain=1),

        advection=ProjectionParameterFunctional('Pe')*ConstantFunction(np.array([1,]), dim_domain=1),

        reaction=ProjectionParameterFunctional('Da')*ConstantFunction(1, dim_domain=1),

        dirichlet_data=ConstantFunction(1, dim_domain=1),

        neumann_data=ConstantFunction(0, dim_domain=1),

        outputs=(('l2_boundary', ExpressionFunction(f'1*(x[..., 0] > (1 - 1e-7))', dim_domain=1)),),
    ),
        initial_data=ConstantFunction(0, dim_domain=1),

        T=T,

        name=f'G0R1_T{T}',
    )


def make_coercivity_estimator(parameter_space):
    # build coercivity constant, works since divergence(advection) = 0 in our case
    # the fixed parameter is chosen to minimize the constant arising in the a posteriori error estimate
    mu_bar = parameter_space.parameters.parse({kk: vv for kk, (vv, _) in parameter_space.ranges.items()})

    # sine the diffusion is non-parametric
    theta_diffusion = ConstantParameterFunctional(1)

    theta_reaction = MinThetaParameterFunctional((ProjectionParameterFunctional('Da'),), mu_bar)

    coercivity_estimator = GenericParameterFunctional(
        mapping=lambda mu: min(theta_diffusion.evaluate(mu), theta_reaction.evaluate(mu)),
        parameters=parameter_space.parameters)

    return coercivity_estimator, mu_bar


parameter_ranges = {
    'diffusion dominated': make_G0R1_problem(1).parameters.space({'Da': (1e-3, 1), 'Pe': (1e-3, 1)}),
    'convection dominated': make_G0R1_problem(1).parameters.space({'Da': (1e-2, 100), 'Pe': (10, 100)}),
    'reaction dominated': make_G0R1_problem(1).parameters.space({'Da': (2, 50), 'Pe': (1e-3, 1)}),
}


T_end = {
    'diffusion dominated': 3.,
    'convection dominated': 0.2,
    'reaction dominated': 1.,
}


def discretize_dune(problem, h, dt,
                    num_values=None,
                    mu_energy_product=None,
                    advection_in_divergence_form=False,
                    ensure_consistent_initial_values=False):
    return discretize_instationary_cg(
        problem,
        diameter=h,
        order=1,
        data_approximation_order=0,
        nt=int(np.ceil(problem.T/dt)),
        num_values=num_values,
        advection_in_divergence_form=advection_in_divergence_form,
        mu_energy_product=mu_energy_product,
        ensure_consistent_initial_values=ensure_consistent_initial_values
    )
